@@ -30,8 +30,8 @@
 
 void on_about (GtkWidget* widget, void* data)
 {
-	char * authors[] = {"Jean Bréfort", NULL};
-	char * artists[] = {"Nestor Diaz", NULL};
+	char * authors[] = {(char*) "Jean Bréfort", NULL};
+	char * artists[] = {(char*) "Nestor Diaz", NULL};
 //	char * documentors[] = {NULL};
 	char license[] = "This program is free software; you can redistribute it and/or\n" 
 		"modify it under the terms of the GNU General Public License as\n"
@@ -53,7 +53,7 @@ void on_about (GtkWidget* widget, void* data)
 					"authors", authors,
 					"artists", artists,
 					"comments", _("GChemPaint is a 2D chemical structures editor for Gnome"),
-					"copyright", _("(C) 2001-2005 by Jean Bréfort"),
+					"copyright", _("Copyright © 2001-2007 by Jean Bréfort"),
 					"license", license,
 					"logo", pixbuf,
 					"icon-name", "gchempaint",
