@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment-atom.cc 
  *
- * Copyright (C) 2003-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2003-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,14 +30,14 @@
 
 gcpFragmentAtom::gcpFragmentAtom(): gcpAtom()
 {
-	SetId(const_cast<char*> ("a1"));
+	SetId ((char*) "a1");
 }
 
 gcpFragmentAtom::gcpFragmentAtom(gcpFragment *fragment, int Z): gcpAtom()
 {
 	m_Fragment = fragment;
 	SetZ(Z);
-	SetId(const_cast<char*> ("a1"));
+	SetId ((char*) "a1");
 }
 
 gcpFragmentAtom::~gcpFragmentAtom() {}
@@ -106,31 +106,31 @@ xmlNodePtr gcpFragmentAtom::Save(xmlDocPtr xml)
 			if (ChargePos) {
 				switch (ChargePos) {
 				case CHARGE_NE:
-					buf = const_cast<char*> ("ne");
+					buf = (char*) "ne";
 					break;
 				case CHARGE_NW:
-					buf = const_cast<char*> ("nw");
+					buf = (char*) "nw";
 					break;
 				case CHARGE_N:
-					buf = const_cast<char*> ("n");
+					buf = (char*) "n";
 					break;
 				case CHARGE_SE:
-					buf = const_cast<char*> ("se");
+					buf = (char*) "se";
 					break;
 				case CHARGE_SW:
-					buf = const_cast<char*> ("sw");
+					buf = (char*) "sw";
 					break;
 				case CHARGE_S:
-					buf= const_cast<char*> ("s");
+					buf= (char*) "s";
 					break;
 				case CHARGE_E:
-					buf = const_cast<char*> ("e");
+					buf = (char*) "e";
 					break;
 				case CHARGE_W:
-					buf = const_cast<char*> ("w");
+					buf = (char*) "w";
 					break;
 				default:
-					buf = const_cast<char*> ("def"); // should not occur
+					buf = (char*) "def"; // should not occur
 				}
 				xmlNewProp (node, (xmlChar*) "charge-position", (xmlChar*) buf);
 			} else {
