@@ -252,7 +252,7 @@ static bool on_delete_event (GtkWidget* widget, GdkEvent *event, gcpPrefsDlg* dl
 }
 
 gcpPrefsDlg::gcpPrefsDlg (gcpApplication *pApp):
-	Dialog (pApp, DATADIR"/gchempaint-unstable/ui/preferences.glade", "preferences", pApp),
+	Dialog (pApp, DATADIR"/gchempaint/ui/preferences.glade", "preferences", pApp),
 	Object (),
 	m_CurTheme (NULL),
 	m_Path (NULL)
@@ -340,7 +340,9 @@ gcpPrefsDlg::gcpPrefsDlg (gcpApplication *pApp):
 	list<string> theme_names = ThemeManager.GetThemesNames ();
 	list<string>::iterator i, iend = theme_names.end ();
 	GtkTreeIter iter, selected, child, grand_child;
-	string &default_name = pApp->GetActiveDocument ()->GetTheme ()->GetName ();
+	string default_name = pApp->GetActiveDocument ()->GetTheme ()->GetName ();
+	if (default_name == "Default")
+			default_name = _("Default");
 	gcpTheme *theme;
 	for (i = theme_names.begin (); i != iend; i++) {
 		theme = ThemeManager.GetTheme (*i);
@@ -459,6 +461,8 @@ void gcpPrefsDlg::OnNewTheme ()
 void gcpPrefsDlg::OnSelectTheme (GtkTreeSelection *selection)
 {
 	char const *Name = gtk_entry_get_text (m_NameEntry);
+	if (!strcmp (Name, _("Default")))
+		Name = "Default";
 	if (m_CurTheme && Name != m_CurTheme->m_Name)
 		OnThemeNameChanged (Name);
 	if (!*Name) {
