@@ -4,9 +4,7 @@
  * GChemPaint library
  * reactant.h 
  *
- * Copyright (C) 2004-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -20,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -43,22 +41,22 @@ public:
 	
 	virtual xmlNodePtr Save (xmlDocPtr xml);
 	virtual bool Load (xmlNodePtr);
-	unsigned GetStoech () {return m_Stoech;}
-	void SetStoech (unsigned Stoech) {m_Stoech = Stoech;}
+	unsigned GetStoich () {return m_Stoich;}
+	void SetStoich (unsigned Stoich) {m_Stoich = Stoich;}
 
-	unsigned GetStoechiometry () {return m_Stoech;}
-	void SetStoechiometry (unsigned coef) {m_Stoech = coef;}
+	unsigned GetStoichiometry () {return m_Stoich;}
+	void SetStoichiometry (unsigned coef) {m_Stoich = coef;}
 	virtual double GetYAlign ();
-	virtual bool BuildContextualMenu (GtkUIManager *UIManager, Object *object);
+	virtual bool BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y);
 	virtual bool OnSignal (SignalId Signal, Object *Child);
 	
-	void AddStoechiometry ();
+	void AddStoichiometry ();
 	Object *GetChild () {return Child;}
-	Object *GetStoechChild () {return Stoechiometry;}
+	Object *GetStoichChild () {return Stoichiometry;}
 
 private:
-	unsigned m_Stoech;	//always positive
-	Object *Stoechiometry;
+	unsigned m_Stoich;	//always positive
+	Object *Stoichiometry;
 	Object *Child;
 };
 
