@@ -4,7 +4,7 @@
  * GChemPaint library
  * reaction-arrow.cc 
  *
- * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -33,11 +33,11 @@
 #include "widgetdata.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-line.h"
-#include <math.h>
+#include <cmath>
 
 gcpReactionArrow::gcpReactionArrow (gcpReaction* react, unsigned Type): gcpArrow (ReactionArrowType)
 {
-	SetId ("ra1");
+	SetId ((char*) "ra1");
 	m_Type = Type;
 	m_Start = m_End = NULL;
 	if (react) react->AddChild (this);
@@ -278,6 +278,10 @@ void gcpReactionArrow::Update (GtkWidget* w)
 			points->coords[3] = (m_y + m_height) * pTheme->GetZoomFactor ();
 			g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "arrow")),
 								"points", points,
+								"width_units", pTheme->GetArrowWidth (),
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								NULL);
 			break;
 		case gcpReversibleArrow:
@@ -289,6 +293,10 @@ void gcpReactionArrow::Update (GtkWidget* w)
 			points->coords[3] = (m_y + m_height) * pTheme->GetZoomFactor () - pTheme->GetArrowDist () / 2 * cos (dAngle);
 			g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "direct")),
 								"points", points,
+								"width_units", pTheme->GetArrowWidth (),
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								NULL);
 			points->coords[2] = m_x * pTheme->GetZoomFactor () + pTheme->GetArrowDist () / 2 * sin (dAngle);
 			points->coords[3] = m_y * pTheme->GetZoomFactor () + pTheme->GetArrowDist () / 2 * cos (dAngle);
@@ -296,6 +304,10 @@ void gcpReactionArrow::Update (GtkWidget* w)
 			points->coords[1] = (m_y + m_height) * pTheme->GetZoomFactor () + pTheme->GetArrowDist () / 2 * cos (dAngle);
 			g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "reverse")),
 								"points", points,
+								"width_units", pTheme->GetArrowWidth (),
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								NULL);
 			break;
 	}
