@@ -27,8 +27,8 @@
 #include "theme.h"
 #include "settings.h"
 #include <glib/gi18n-lib.h>
-#include <math.h>
 #include <sys/stat.h>
+#include <cmath>
 
 double DefaultBondLength = 140.;
 double DefaultBondAngle = 120.;
@@ -599,13 +599,13 @@ bool gcpTheme::Save (xmlDocPtr xml)
 	buf = NULL;
 	switch (m_FontStyle) {
 	case PANGO_STYLE_NORMAL:
-		buf = "normal";
+		buf = (char*) "normal";
 		break;
 	case PANGO_STYLE_OBLIQUE:
-		buf = "oblique";
+		buf = (char*) "oblique";
 		break;
 	case PANGO_STYLE_ITALIC:
-		buf = "italic";
+		buf = (char*) "italic";
 		break;
 	}
 	if (buf)
@@ -613,25 +613,25 @@ bool gcpTheme::Save (xmlDocPtr xml)
 	buf = NULL;
 	switch (m_FontWeight) {
 	case PANGO_WEIGHT_ULTRALIGHT:
-		buf = "ultra-light";
+		buf = (char*) "ultra-light";
 		break;
 	case PANGO_WEIGHT_LIGHT:
-		buf = "light";
+		buf = (char*) "light";
 		break;
 	case PANGO_WEIGHT_NORMAL:
-		buf = "normal";
+		buf = (char*) "normal";
 		break;
 	case PANGO_WEIGHT_SEMIBOLD:
-		buf = "semi-bold";
+		buf = (char*) "semi-bold";
 		break;
 	case PANGO_WEIGHT_BOLD:
-		buf = "bold";
+		buf = (char*) "bold";
 		break;
 	case PANGO_WEIGHT_ULTRABOLD:
-		buf = "ultra-bold";
+		buf = (char*) "ultra-bold";
 		break;
 	case PANGO_WEIGHT_HEAVY:
-		buf = "heavy";
+		buf = (char*) "heavy";
 		break;
 	}
 	if (buf)
@@ -640,31 +640,31 @@ bool gcpTheme::Save (xmlDocPtr xml)
 	buf = NULL;
 	switch (m_FontStretch) {
 	case PANGO_STRETCH_ULTRA_CONDENSED:
-		buf = "ultra-condensed";
+		buf = (char*) "ultra-condensed";
 		break;
 	case PANGO_STRETCH_EXTRA_CONDENSED:
-		buf = "extra-condensed";
+		buf = (char*) "extra-condensed";
 		break;
 	case PANGO_STRETCH_CONDENSED:
-		buf = "condensed";
+		buf = (char*) "condensed";
 		break;
 	case PANGO_STRETCH_SEMI_CONDENSED:
-		buf = "semi-condensed";
+		buf = (char*) "semi-condensed";
 		break;
 	case PANGO_STRETCH_NORMAL:
-		buf = "normal";
+		buf = (char*) "normal";
 		break;
 	case PANGO_STRETCH_SEMI_EXPANDED:
-		buf = "semi-expanded";
+		buf = (char*) "semi-expanded";
 		break;
 	case PANGO_STRETCH_EXPANDED:
-		buf = "expanded";
+		buf = (char*) "expanded";
 		break;
 	case PANGO_STRETCH_EXTRA_EXPANDED:
-		buf = "extra-expanded";
+		buf = (char*) "extra-expanded";
 		break;
 	case PANGO_STRETCH_ULTRA_EXPANDED:
-		buf = "ultra-expanded";
+		buf = (char*) "ultra-expanded";
 		break;
 	}
 	if (buf)
@@ -676,13 +676,13 @@ bool gcpTheme::Save (xmlDocPtr xml)
 	buf = NULL;
 	switch (m_TextFontStyle) {
 	case PANGO_STYLE_NORMAL:
-		buf = "normal";
+		buf = (char*) "normal";
 		break;
 	case PANGO_STYLE_OBLIQUE:
-		buf = "oblique";
+		buf = (char*) "oblique";
 		break;
 	case PANGO_STYLE_ITALIC:
-		buf = "italic";
+		buf = (char*) "italic";
 		break;
 	}
 	if (buf)
@@ -690,25 +690,25 @@ bool gcpTheme::Save (xmlDocPtr xml)
 	buf = NULL;
 	switch (m_TextFontWeight) {
 	case PANGO_WEIGHT_ULTRALIGHT:
-		buf = "ultra-light";
+		buf = (char*) "ultra-light";
 		break;
 	case PANGO_WEIGHT_LIGHT:
-		buf = "light";
+		buf = (char*) "light";
 		break;
 	case PANGO_WEIGHT_NORMAL:
-		buf = "normal";
+		buf = (char*) "normal";
 		break;
 	case PANGO_WEIGHT_SEMIBOLD:
-		buf = "semi-bold";
+		buf = (char*) "semi-bold";
 		break;
 	case PANGO_WEIGHT_BOLD:
-		buf = "bold";
+		buf = (char*) "bold";
 		break;
 	case PANGO_WEIGHT_ULTRABOLD:
-		buf = "ultra-bold";
+		buf = (char*) "ultra-bold";
 		break;
 	case PANGO_WEIGHT_HEAVY:
-		buf = "heavy";
+		buf = (char*) "heavy";
 		break;
 	}
 	if (buf)
@@ -717,31 +717,31 @@ bool gcpTheme::Save (xmlDocPtr xml)
 	buf = NULL;
 	switch (m_TextFontStretch) {
 	case PANGO_STRETCH_ULTRA_CONDENSED:
-		buf = "ultra-condensed";
+		buf = (char*) "ultra-condensed";
 		break;
 	case PANGO_STRETCH_EXTRA_CONDENSED:
-		buf = "extra-condensed";
+		buf = (char*) "extra-condensed";
 		break;
 	case PANGO_STRETCH_CONDENSED:
-		buf = "condensed";
+		buf = (char*) "condensed";
 		break;
 	case PANGO_STRETCH_SEMI_CONDENSED:
-		buf = "semi-condensed";
+		buf = (char*) "semi-condensed";
 		break;
 	case PANGO_STRETCH_NORMAL:
-		buf = "normal";
+		buf = (char*) "normal";
 		break;
 	case PANGO_STRETCH_SEMI_EXPANDED:
-		buf = "semi-expanded";
+		buf = (char*) "semi-expanded";
 		break;
 	case PANGO_STRETCH_EXPANDED:
-		buf = "expanded";
+		buf = (char*) "expanded";
 		break;
 	case PANGO_STRETCH_EXTRA_EXPANDED:
-		buf = "extra-expanded";
+		buf = (char*) "extra-expanded";
 		break;
 	case PANGO_STRETCH_ULTRA_EXPANDED:
-		buf = "ultra-expanded";
+		buf = (char*) "ultra-expanded";
 		break;
 	}
 	if (buf)
