@@ -4,7 +4,7 @@
  * GChemPaint library
  * view.h 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,7 +29,7 @@
 #include <list>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomecanvas/libgnomecanvas.h>
-#include <libgcpcanvas/gcp-canvas-rich-text.h>
+#include <libgcpcanvas/gcp-canvas-pango.h>
 #include <gcu/macros.h>
 #include "atom.h"
 #include "bond.h"
@@ -66,11 +66,8 @@ public:
 	double GetFontHeight() {return m_dFontHeight;}
 	gchar* GetFontName() {return m_sFontName;}
 	gchar* GetSmallFontName() {return m_sSmallFontName;}
-	gchar* GetTinyFontName() {return m_sTinyFontName;}
 	PangoFontDescription* GetPangoFontDesc() {return m_PangoFontDesc;}
 	PangoFontDescription* GetPangoSmallFontDesc() {return m_PangoSmallFontDesc;}
-	PangoFontDescription* GetPangoTinyFontDesc() {return m_PangoTinyFontDesc;}
-	void UpdateLabel(const gchar* label);
 	void OnDeleteSelection(GtkWidget* w);
 	void OnCopySelection(GtkWidget* w, GtkClipboard* clipboard);
 	void OnPasteSelection(GtkWidget* w, GtkClipboard* clipboard);
@@ -79,15 +76,18 @@ public:
 	bool OnKeyRelease(GtkWidget* w, GdkEventKey* event);
 	bool OnSize (GtkWidget *w, int width, int height);
 	void UpdateSize(double x1, double y1, double x2, double y2);
-	void SetGnomeCanvasRichTextActive(GnomeCanvasRichTextExt* item);
+	void SetGnomeCanvasPangoActive (GnomeCanvasPango* item);
 	bool PrepareUnselect();
 	void OnReceive(GtkClipboard* clipboard, GtkSelectionData* selection_data);
 	void OnSelectAll();
 	bool IsEmbedded() {return m_bEmbedded;}
 	int GetNbWidgets () {return m_Widgets.size ();}
-	void ExportImage (const char* filename, const char* type);
+	void ExportImage (string const &filename, const char* type, int resolution = -1);
+	xmlDocPtr BuildSVG ();
 	void EnsureSize ();
 	void Zoom (double zoom);
+	void ShowCursor (bool show);
+	void UpdateTheme ();
 
 	//Implementation
 private:
@@ -96,17 +96,17 @@ private:
 	GtkWidget* m_pWidget;
 	std::list<GtkWidget*>m_Widgets;
 	PangoContext* m_PangoContext;
-	PangoFontDescription* m_PangoFontDesc, *m_PangoSmallFontDesc, *m_PangoTinyFontDesc;
+	PangoFontDescription* m_PangoFontDesc, *m_PangoSmallFontDesc;
 	double m_dFontHeight;
-	gchar* m_sFontName, *m_sSmallFontName, *m_sTinyFontName;
+	gchar* m_sFontName, *m_sSmallFontName;
 	int m_width, m_height;
 	double m_lastx, m_lasty;
-	GnomeCanvasRichTextExt* m_ActiveRichText;
 	bool m_bEmbedded;
 	GtkUIManager *m_UIManager;
 	bool m_Dragging;
 
 GCU_RO_PROP (double, BaseLineOffset)
+GCU_RO_PROP (GnomeCanvasPango*, ActiveRichText);
 };
 
 bool on_event(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget);
