@@ -37,14 +37,14 @@
 #include "lib/theme.h"
 #include <glib/gi18n-lib.h>
 #include <gconf/gconf-client.h>
-#include <math.h>
+#include <cmath>
 
 static char* ToolNames[] = {
-	"SimpleArrow",
-	"ReversibleArrow",
-	"ReversibleArrow",
-	"DoubleHeadedArrow",
-	"RetrosynthesisArrow"
+	(char*) "SimpleArrow",
+	(char*) "ReversibleArrow",
+	(char*) "ReversibleArrow",
+	(char*) "DoubleHeadedArrow",
+	(char*) "RetrosynthesisArrow"
 };
 
 static void on_default (GtkToggleButton *button)
