@@ -29,20 +29,20 @@
 #include "lib/application.h"
 #include "lib/theme.h"
 #include <glib/gi18n-lib.h>
-#include <math.h>
+#include <cmath>
 #include <list>
 #include <vector>
 
 using namespace std;
 
 static char* ToolNames [] = {
-	"Cycle3",
-	"Cycle4",
-	"Cycle5",
-	"Cycle6",
-	"Cycle7",
-	"Cycle8",
-	"CycleN",
+	(char*) "Cycle3",
+	(char*) "Cycle4",
+	(char*) "Cycle5",
+	(char*) "Cycle6",
+	(char*) "Cycle7",
+	(char*) "Cycle8",
+	(char*) "CycleN",
 };
 
 gcpCycleTool::gcpCycleTool(gcpApplication *App, unsigned char size): gcpTool(App, ToolNames[size - 3])
