@@ -2,7 +2,7 @@
  * GChemPaint selection plugin
  * groupdlg.cc
  *
- * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -39,8 +39,12 @@ static void on_space_toggled (gcpGroupDlg *dlg)
 }
 
 gcpGroupDlg::gcpGroupDlg (gcpDocument *Doc, gcpGroup *group):
-	Dialog (Doc->GetApplication (), DATADIR"/gchempaint-unstable/ui/group.glade", "group")
+	Dialog (Doc->GetApplication (), DATADIR"/gchempaint/ui/group.glade", "group", Doc)
 {
+	if (!xml) {
+		delete this;
+		return;
+	}
 	m_Group = group;
 	m_Doc = Doc;
 	m_Data = (gcpWidgetData*) g_object_get_data (G_OBJECT (Doc->GetWidget ()), "data");
