@@ -45,7 +45,7 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	xmlIndentTreeOutput = true;
 	xmlKeepBlanksDefault (0);
 	if (dir) {
-		while (name = g_dir_read_name (dir)) {
+		while ((name = g_dir_read_name (dir))) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			name = g_strconcat (DATADIR"/gchempaint/templates/", name, NULL);
 			doc = xmlParseFile (name);
@@ -60,7 +60,7 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	dir = g_dir_open (path, 0, NULL);
 	if (dir) {
 		bool is_user_templates;
-		while (name = g_dir_read_name (dir)) {
+		while ((name = g_dir_read_name (dir))) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			is_user_templates = !strcmp (name, "templates.xml");
 			name = g_strconcat (path, "/", name, NULL);
@@ -119,7 +119,6 @@ static const char *ui_description =
 
 void gcpTemplatesPlugin::Populate (gcpApplication* App)
 {
-	App->AddUI (DATADIR"/gchempaint-unstable/ui/templates.xml");
 	App->AddActions (entries, G_N_ELEMENTS (entries), ui_description, NULL);
 	new gcpTemplateTool (App);
 	new gcpTemplateTree (App);
